﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HEAL.Attic;

namespace HeuristicLab.Problems.MetaOptimization {
  [Item("AlgorithmSubScopesCreator", "An operator which ...")]
  [StorableType("4569682A-267C-4471-93F4-5CB202E36B0B")]
  public class AlgorithmSubScopesCreator : SingleSuccessorOperator {
    #region Parameter properties
    public ILookupParameter<TypeValue> AlgorithmTypeParameter {
      get { return (ILookupParameter<TypeValue>)Parameters[MetaOptimizationProblem.AlgorithmTypeParameterName]; }
    }
    public ILookupParameter<IItemList<IProblem>> ProblemsParameter {
      get { return (ILookupParameter<IItemList<IProblem>>)Parameters[MetaOptimizationProblem.ProblemsParameterName]; }
    }
    public ILookupParameter<ParameterConfigurationTree> ParameterConfigurationParameter {
      get { return (ILookupParameter<ParameterConfigurationTree>)Parameters["ParameterConfigurationTree"]; }
    }
    public LookupParameter<IntValue> RepetitionsParameter {
      get { return (LookupParameter<IntValue>)Parameters[MetaOptimizationProblem.RepetitionsParameterName]; }
    }
    private ScopeParameter CurrentScopeParameter {
      get { return (ScopeParameter)Parameters["CurrentScope"]; }
    }
    public IScope CurrentScope {
      get { return CurrentScopeParameter.ActualValue; }
    }
    #endregion

    [StorableConstructor]
    protected AlgorithmSubScopesCreator(StorableConstructorFlag _) : base(_) { }
    public AlgorithmSubScopesCreator()
      : base() {
      Parameters.Add(new LookupParameter<TypeValue>(MetaOptimizationProblem.AlgorithmTypeParameterName, ""));
      Parameters.Add(new LookupParameter<IItemList<IProblem>>(MetaOptimizationProblem.ProblemsParameterName, ""));
      Parameters.Add(new LookupParameter<ParameterConfigurationTree>("ParameterConfigurationTree", ""));
      Parameters.Add(new LookupParameter<IntValue>(MetaOptimizationProblem.RepetitionsParameterName, "Number of evaluations on one problem."));
      Parameters.Add(new ScopeParameter("CurrentScope", "The current scope whose sub-scopes represent the parents."));
    }
    protected AlgorithmSubScopesCreator(AlgorithmSubScopesCreator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AlgorithmSubScopesCreator(this, cloner);
    }

    public override IOperation Apply() {
      ParameterConfigurationTree parameterConfiguration = ParameterConfigurationParameter.ActualValue;
      IItemList<IProblem> problems = ProblemsParameter.ActualValue;
      int repetitions = RepetitionsParameter.ActualValue.Value;
      Type algorithmType = AlgorithmTypeParameter.ActualValue.Value;

      for (int i = 0; i < repetitions; i++) {
        for (int j = 0; j < problems.Count; j++) {
          IScope child = new Scope(string.Format("Problem {0}, Repetition {1}", j, i));
          var algorithm = MetaOptimizationUtil.CreateParameterizedAlgorithmInstance(parameterConfiguration, algorithmType, (IProblem)problems[j].Clone());
          child.Variables.Add(new Variable("Algorithm", algorithm));
          child.Variables.Add(new Variable("ProblemIndex", new IntValue(j)));
          child.Variables.Add(new Variable("RepetitionIndex", new IntValue(i)));
          CurrentScope.SubScopes.Add(child);
        }
      }
      return base.Apply();
    }
  }
}